<?php

namespace Drupal\menu_block\Plugin\Block;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an extended Menu block.
 *
 * @Block(
 *   id = "multiple_menu_block",
 *   admin_label = @Translation("Multiple Menu block"),
 *   category = @Translation("Menus"),
 *   deriver = "Drupal\menu_block\Plugin\Derivative\MultipleMenuBlock",
 *   forms = {
 *     "settings_tray" = "\Drupal\system\Form\SystemMenuOffCanvasForm",
 *   },
 * )
 */
class MultipleMenuBlock extends MenuBlock implements ContainerFactoryPluginInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * Creates a MenuBlockMultiMenu instance.
   *
   * @param array $configuration
   * @param string $plugin_id
   * @param array $plugin_definition
   * @param EntityTypeManagerInterface $entity_type_manager
   *   The entity manager.
   * @param \Drupal\Core\Menu\MenuLinkManagerInterface $menu_link_manager
   *   The menu link manager.
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_tree
   *   The menu tree service.
   * @param \Drupal\Core\Menu\MenuActiveTrail $menu_active_trail
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, MenuLinkTreeInterface $menu_tree) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $menu_tree);
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('menu.link_tree'));
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    // Load and display the menus lists.
    $form['menus'] = [
      '#type' => 'details',
      '#title' => $this->t('Selected menus'),
      '#open' => TRUE,
    ];
    $menus = $this->entityTypeManager->getStorage('menu')->loadMultiple();

    $options = [];
    foreach ($menus as $key => $option) {
      $options[$option->id()] = [
        'id' => $option->id(),
        'label' => $option->label(),
      ];
    }
    $form['menus']['selected_menus'] = [
      '#type' => 'tableselect',
      '#caption' => 'Menus',
      '#header' => [
        'label' => $this->t('Label'),
        'id' => $this->t('ID'),
      ],
      '#empty' => $this->t('No items.'),
      '#options' => $options,
      '#multiple' => TRUE,
      '#default_value' => $this->configuration['selected_menus'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['selected_menus'] = $form_state->getValue('menus')['selected_menus'];
    parent::blockSubmit($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function build($menu_name = '') {
    // List possible menus for display.
    $selected_menus = $this->configuration['selected_menus'];
    foreach ($selected_menus as $selected_menu) {
      // Get menu link for this page.
      if ($menu_link_ids = $this->menuActiveTrail->getActiveTrailIds($selected_menu)) {
        $menu_link_id = reset($menu_link_ids);
        if (!empty($menu_link_id)) {
          $build = parent::build($selected_menu);
          $build['#cache']['max-age'] = 0;
          return $build;
        }
      }
    }
  }

}
